@@ -1,3 +1,5 @@
+from dataclasses import field
+
 import jax
 import pytest
 from jax import numpy as jnp
@@ -16,27 +18,8 @@ class Linear:
         )
         self.bias = jnp.ones((1, out_dim))
 
-    def __call__(self, x):
-        return x @ self.weight + self.bias
-
-
-# def test_op():
-#     @pytc.treeclass(op=False)
-#     class StackedLinear:
-#         def __init__(self, key, in_dim, out_dim, hidden_dim):
-#             keys = jax.random.split(key, 3)
-
-#             # Declaring l1,l2,l3 as dataclass_fields is optional
-#             # as l1,l2,l3 are Linear class that is wrapped with @pytc.treeclass
-#             self.l1 = Linear(key=keys[0], in_dim=in_dim, out_dim=hidden_dim)
-#             self.l2 = Linear(key=keys[1], in_dim=hidden_dim, out_dim=hidden_dim)
-#             self.l3 = Linear(key=keys[2], in_dim=hidden_dim, out_dim=out_dim)
-
-#     with pytest.raises(TypeError):
-#         model = StackedLinear(
-#             key=jax.random.PRNGKey(0), in_dim=2, out_dim=2, hidden_dim=2
-#         )
-#         model + model
+    # def __call__(self, x):
+    #     return x @ self.weight + self.bias
 
 
 def test_field_only():
@@ -70,7 +53,7 @@ def test_field_only():
     model = StackedLinear(key=jax.random.PRNGKey(0), in_dim=2, out_dim=2, hidden_dim=2)
     model.__tree_fields__
 
-    assert "l1" in model.__dataclass_fields__
+    assert "l1" in model.__treeclass_fields__
 
     @pytc.treeclass(field_only=False)
     class StackedLinear:
@@ -94,7 +77,7 @@ def test_field_only():
 
     @pytc.treeclass(field_only=True)
     class StackedLinear:
-        l4: Linear = pytc.static_field(default=1)
+        l4: Linear = field(default=pytc.static_value(1))
         l5: Linear
 
         def __init__(self, key, in_dim, out_dim, hidden_dim):
@@ -106,3 +89,12 @@ def test_field_only():
             key=jax.random.PRNGKey(0), in_dim=2, out_dim=2, hidden_dim=2
         )
         model.__tree_fields__
+
+
+def test_hash():
+    @pytc.treeclass
+    class T:
+        a: jnp.ndarray
+
+    with pytest.raises(TypeError):
+        hash(T(jnp.array([1, 2, 3])))
