@@ -4,13 +4,15 @@ from dataclasses import field
 
 import jax
 import jax.numpy as jnp
+import numpy as np
 import pytest
 
-from pytreeclass import treeclass
-from pytreeclass.src.tree_util import is_treeclass_equal
+import pytreeclass as pytc
+from pytreeclass.src.tree_indexer import _at_apply, _at_get, _at_reduce, _at_set
+from pytreeclass.src.tree_util import is_treeclass_equal, static_value
 
 
-@treeclass
+@pytc.treeclass
 class Test:
     a: float
     b: float
@@ -20,13 +22,13 @@ class Test:
 
 
 def test_getter_by_val():
-    @treeclass
+    @pytc.treeclass
     class level1:
         a: int
         b: int
         c: int
 
-    @treeclass
+    @pytc.treeclass
     class level2:
         d: level1
         e: level1
@@ -44,6 +46,14 @@ def test_getter_by_val():
 
     assert is_treeclass_equal(B, C)
 
+    B = A.at[(A > 0) & (A < 5)].get()
+    C = level2(
+        d=level1(a=1, b=None, c=jnp.array([1, 2, 3, 4])),
+        e=level1(a=2, b=None, c=jnp.array([])),
+    )
+
+    assert is_treeclass_equal(B, C)
+
     B = A.at[A == 0].get()
     C = level2(
         d=level1(a=None, b=None, c=jnp.array([])),
@@ -60,39 +70,42 @@ def test_getter_by_val():
 
 
 def test_getter_by_param():
-    A = Test(10, 20, 30, jnp.array([1, 2, 3, 4, 5]), "A")
+
+    A = Test(10, 20, 30, jnp.array([1, 2, 3, 4, 5]), static_value("A"))
 
     B = A.at[A == "a"].get(array_as_leaves=False)
-    assert is_treeclass_equal(B, Test(10, None, None, None, "A"))
+    assert is_treeclass_equal(B, Test(10, None, None, None, static_value("A")))
 
     B = A.at[(A == "a") | (A == "b")].get(array_as_leaves=False)
-    assert is_treeclass_equal(B, Test(10, 20, None, None, "A"))
+    assert is_treeclass_equal(B, Test(10, 20, None, None, static_value("A")))
 
     B = A.at[A == ""].get(array_as_leaves=False)
-    assert is_treeclass_equal(B, Test(None, None, None, None, "A"))
+    assert is_treeclass_equal(B, Test(None, None, None, None, static_value("A")))
 
     B = A.at[(A == "a") | (A == "b") | (A == "c")].get(array_as_leaves=False)
-    assert is_treeclass_equal(B, Test(10, 20, 30, None, "A"))
+    assert is_treeclass_equal(B, Test(10, 20, 30, None, static_value("A")))
 
     B = A.at[(A == "a") | (A == "b") | (A == "c") | (A == "d")].get(
         array_as_leaves=False
     )
-    assert is_treeclass_equal(B, Test(10, 20, 30, jnp.array([1, 2, 3, 4, 5]), "A"))
+    assert is_treeclass_equal(
+        B, Test(10, 20, 30, jnp.array([1, 2, 3, 4, 5]), static_value("A"))
+    )
 
-    @treeclass
+    @pytc.treeclass
     class L0:
         a: int = 1
         b: int = 2
         c: int = 3
 
-    @treeclass
+    @pytc.treeclass
     class L1:
         a: int = 1
         b: int = 2
         c: int = 3
         d: L0 = L0()
 
-    @treeclass
+    @pytc.treeclass
     class L2:
         a: int = 10
         b: int = 20
@@ -115,7 +128,7 @@ def test_getter_by_param():
 
 
 def test_getter_by_metadata():
-    @treeclass
+    @pytc.treeclass
     class Test:
         a: float = field(metadata={"name": "a", "unit": "m"})
         b: float = field(metadata={"name": "b", "unit": "m"})
@@ -123,21 +136,21 @@ def test_getter_by_metadata():
         d: jnp.ndarray = field(metadata={"name": "d", "unit": "m"})
         name: str
 
-    A = Test(10, 20, 30, jnp.array([1, 2, 3, 4, 5]), "A")
+    A = Test(10, 20, 30, jnp.array([1, 2, 3, 4, 5]), static_value("A"))
 
     B = A.at[A == {"name": "a"}].get(array_as_leaves=False)
-    assert is_treeclass_equal(B, Test(10, None, None, None, "A"))
+    assert is_treeclass_equal(B, Test(10, None, None, None, static_value("A")))
 
     B = A.at[(A == {"name": "a"}) | (A == {"name": "b"})].get(array_as_leaves=False)
-    assert is_treeclass_equal(B, Test(10, 20, None, None, "A"))
+    assert is_treeclass_equal(B, Test(10, 20, None, None, static_value("A")))
 
     B = A.at[A == {"": ""}].get(array_as_leaves=False)
-    assert is_treeclass_equal(B, Test(None, None, None, None, "A"))
+    assert is_treeclass_equal(B, Test(None, None, None, None, static_value("A")))
 
     B = A.at[(A == {"name": "a"}) | (A == {"name": "b"}) | (A == {"name": "c"})].get(
         array_as_leaves=False
     )
-    assert is_treeclass_equal(B, Test(10, 20, 30, None, "A"))
+    assert is_treeclass_equal(B, Test(10, 20, 30, None, static_value("A")))
 
     B = A.at[
         (A == {"name": "a"})
@@ -145,22 +158,24 @@ def test_getter_by_metadata():
         | (A == {"name": "c"})
         | (A == {"name": "d"})
     ].get(array_as_leaves=False)
-    assert is_treeclass_equal(B, Test(10, 20, 30, jnp.array([1, 2, 3, 4, 5]), "A"))
+    assert is_treeclass_equal(
+        B, Test(10, 20, 30, jnp.array([1, 2, 3, 4, 5]), static_value("A"))
+    )
 
-    @treeclass
+    @pytc.treeclass
     class L0:
         a: int = field(default=1, metadata={"name": "a", "unit": "m"})
         b: int = 2
         c: int = 3
 
-    @treeclass
+    @pytc.treeclass
     class L1:
         a: int = field(default=1, metadata={"name": "a", "unit": "m"})
         b: int = 2
         c: int = 3
         d: L0 = L0()
 
-    @treeclass
+    @pytc.treeclass
     class L2:
         a: int = field(default=10, metadata={"name": "a", "unit": "m"})
         b: int = 20
@@ -174,13 +189,13 @@ def test_getter_by_metadata():
 
 
 def test_setter_by_val():
-    @treeclass
+    @pytc.treeclass
     class level1:
         a: int
         b: int
         c: int
 
-    @treeclass
+    @pytc.treeclass
     class level2:
         d: level1
         e: level1
@@ -198,7 +213,7 @@ def test_setter_by_val():
 
     assert is_treeclass_equal(B, C)
 
-    A = Test(10, 20, 30, jnp.array([1, 2, 3, 4, 5]), "A")
+    A = Test(10, 20, 30, jnp.array([1, 2, 3, 4, 5]), static_value("A"))
 
     with pytest.raises(AssertionError):
         B = A.at[A].set(0)
@@ -206,20 +221,20 @@ def test_setter_by_val():
     with pytest.raises(NotImplementedError):
         B = A.at[0].set(0)
 
-    @treeclass
+    @pytc.treeclass
     class L0:
         a: int = 1
         b: int = 2
         c: int = 3
 
-    @treeclass
+    @pytc.treeclass
     class L1:
         a: int = 1
         b: int = 2
         c: int = 3
         d: L0 = L0()
 
-    @treeclass
+    @pytc.treeclass
     class L2:
         a: int = 10
         b: int = 20
@@ -233,20 +248,26 @@ def test_setter_by_val():
 
 
 def test_setter_by_param():
-    A = Test(10, 20, 30, jnp.array([1, 2, 3, 4, 5]), "A")
+    A = Test(10, 20, 30, jnp.array([1, 2, 3, 4, 5]), static_value("A"))
 
     B = A.at[A == "a"].set(0)
-    assert is_treeclass_equal(B, Test(0, 20, 30, jnp.array([1, 2, 3, 4, 5]), "A"))
+    assert is_treeclass_equal(
+        B, Test(0, 20, 30, jnp.array([1, 2, 3, 4, 5]), static_value("A"))
+    )
 
     B = A.at[(A == "a") | (A == "b")].set(0)
-    assert is_treeclass_equal(B, Test(0, 0, 30, jnp.array([1, 2, 3, 4, 5]), "A"))
+    assert is_treeclass_equal(
+        B, Test(0, 0, 30, jnp.array([1, 2, 3, 4, 5]), static_value("A"))
+    )
 
     B = A.at[(A == "a") | (A == "b") | (A == "c")].set(0)
-    assert is_treeclass_equal(B, Test(0, 0, 0, jnp.array([1, 2, 3, 4, 5]), "A"))
+    assert is_treeclass_equal(
+        B, Test(0, 0, 0, jnp.array([1, 2, 3, 4, 5]), static_value("A"))
+    )
 
 
 def test_setter_by_metadata():
-    @treeclass
+    @pytc.treeclass
     class Test:
         a: float = field(metadata={"name": "a", "unit": "m"})
         b: float = field(metadata={"name": "b", "unit": "m"})
@@ -254,23 +275,31 @@ def test_setter_by_metadata():
         d: jnp.ndarray = field(metadata={"name": "d", "unit": "m"})
         name: str
 
-    A = Test(10, 20, 30, jnp.array([1, 2, 3, 4, 5]), "A")
+    A = Test(10, 20, 30, jnp.array([1, 2, 3, 4, 5]), static_value("A"))
 
     B = A.at[A == {"name": "a"}].set(100, array_as_leaves=False)
-    assert is_treeclass_equal(B, Test(100, 20, 30, jnp.array([1, 2, 3, 4, 5]), "A"))
+    assert is_treeclass_equal(
+        B, Test(100, 20, 30, jnp.array([1, 2, 3, 4, 5]), static_value("A"))
+    )
 
     B = A.at[(A == {"name": "a"}) | (A == {"name": "b"})].set(
         100, array_as_leaves=False
     )
-    assert is_treeclass_equal(B, Test(100, 100, 30, jnp.array([1, 2, 3, 4, 5]), "A"))
+    assert is_treeclass_equal(
+        B, Test(100, 100, 30, jnp.array([1, 2, 3, 4, 5]), static_value("A"))
+    )
 
     B = A.at[A == {"": ""}].set(100, array_as_leaves=False)
-    assert is_treeclass_equal(B, Test(10, 20, 30, jnp.array([1, 2, 3, 4, 5]), "A"))
+    assert is_treeclass_equal(
+        B, Test(10, 20, 30, jnp.array([1, 2, 3, 4, 5]), static_value("A"))
+    )
 
     B = A.at[(A == {"name": "a"}) | (A == {"name": "b"}) | (A == {"name": "c"})].set(
         100, array_as_leaves=False
     )
-    assert is_treeclass_equal(B, Test(100, 100, 100, jnp.array([1, 2, 3, 4, 5]), "A"))
+    assert is_treeclass_equal(
+        B, Test(100, 100, 100, jnp.array([1, 2, 3, 4, 5]), static_value("A"))
+    )
 
     B = A.at[
         (A == {"name": "a"})
@@ -278,22 +307,22 @@ def test_setter_by_metadata():
         | (A == {"name": "c"})
         | (A == {"name": "d"})
     ].set(100, array_as_leaves=False)
-    assert is_treeclass_equal(B, Test(100, 100, 100, 100, "A"))
+    assert is_treeclass_equal(B, Test(100, 100, 100, 100, static_value("A")))
 
-    @treeclass
+    @pytc.treeclass
     class L0:
         a: int = field(default=1, metadata={"name": "a", "unit": "m"})
         b: int = 2
         c: int = 3
 
-    @treeclass
+    @pytc.treeclass
     class L1:
         a: int = field(default=1, metadata={"name": "a", "unit": "m"})
         b: int = 2
         c: int = 3
         d: L0 = L0()
 
-    @treeclass
+    @pytc.treeclass
     class L2:
         a: int = field(default=10, metadata={"name": "a", "unit": "m"})
         b: int = 20
@@ -305,9 +334,16 @@ def test_setter_by_metadata():
     rhs = L2(100, 20, 30, L1(100, 2, 3, L0(100, 2, 3)))
     assert is_treeclass_equal(lhs, rhs)
 
+    @pytc.treeclass
+    class T:
+        a: jnp.ndarray
+
+    t = T(True)
+    assert is_treeclass_equal(t.at[t == bool].set(False), T(a=False))
+
 
 def test_apply_and_its_derivatives():
-    @treeclass
+    @pytc.treeclass
     class A:
         a: int
         b: int
@@ -320,28 +356,40 @@ def test_apply_and_its_derivatives():
     rhs = init.at[init == init].apply(lambda x: x**2)
     assert is_treeclass_equal(lhs, rhs)
 
+    lhs = A(1, 4, jnp.array([1, 4, 9, 16, 25]))
+    rhs = init.at[...].apply(lambda x: x**2)
+    assert is_treeclass_equal(lhs, rhs)
+
     lhs = A(2, 3, jnp.array([2, 3, 4, 5, 6]))
     rhs = init.at[init == init].apply(lambda x: x + 1)
     assert is_treeclass_equal(lhs, rhs)
 
+    lhs = A(2, 3, jnp.array([2, 3, 4, 5, 6]))
+    rhs = init.at[...].apply(lambda x: x + 1)
+    assert is_treeclass_equal(lhs, rhs)
+
     lhs = A(20, 30, jnp.array([20, 30, 40, 50, 60]))
     rhs = init.at[init == init].apply(lambda x: (x + 1) * 10)
     assert is_treeclass_equal(lhs, rhs)
 
-    @treeclass
+    lhs = A(20, 30, jnp.array([20, 30, 40, 50, 60]))
+    rhs = init.at[...].apply(lambda x: (x + 1) * 10)
+    assert is_treeclass_equal(lhs, rhs)
+
+    @pytc.treeclass
     class L0:
         a: int = 1
         b: int = 2
         c: int = 3
 
-    @treeclass
+    @pytc.treeclass
     class L1:
         a: int = 1
         b: int = 2
         c: int = 3
         d: L0 = L0()
 
-    @treeclass
+    @pytc.treeclass
     class L2:
         a: int = 10
         b: int = 20
@@ -357,10 +405,18 @@ def test_apply_and_its_derivatives():
     rhs = init.at[init == init].add(1)
     assert is_treeclass_equal(lhs, rhs)
 
+    lhs = A(2, 3, jnp.array([2, 3, 4, 5, 6]))
+    rhs = init.at[...].add(1)
+    assert is_treeclass_equal(lhs, rhs)
+
     lhs = A(0.5, 1.0, jnp.array([0.5, 1.0, 1.5, 2.0, 2.5]))
     rhs = init.at[init == init].divide(2.0)
     assert is_treeclass_equal(lhs, rhs)
 
+    lhs = A(0.5, 1.0, jnp.array([0.5, 1.0, 1.5, 2.0, 2.5]))
+    rhs = init.at[...].divide(2.0)
+    assert is_treeclass_equal(lhs, rhs)
+
     lhs = A(1, 1, jnp.array([1, 1, 1, 1, 1]))
     rhs = init.at[init == init].min(1)
     assert is_treeclass_equal(lhs, rhs)
@@ -369,10 +425,18 @@ def test_apply_and_its_derivatives():
     rhs = init.at[init == init].max(4)
     assert is_treeclass_equal(lhs, rhs)
 
+    lhs = A(4, 4, jnp.array([4, 4, 4, 4, 5]))
+    rhs = init.at[...].max(4)
+    assert is_treeclass_equal(lhs, rhs)
+
     lhs = A(1, 4, jnp.array([1, 4, 9, 16, 25]))
     rhs = init.at[init == init].power(2)
     assert is_treeclass_equal(lhs, rhs)
 
+    lhs = A(1, 4, jnp.array([1, 4, 9, 16, 25]))
+    rhs = init.at[...].power(2)
+    assert is_treeclass_equal(lhs, rhs)
+
     lhs = A(1, 2, jnp.array([1, 2, 3, 16, 25]))
     rhs = init.at[init > 3].apply(lambda x: x**2)
     assert is_treeclass_equal(lhs, rhs)
@@ -430,7 +494,28 @@ def test_apply_and_its_derivatives():
     rhs = init.at[init == "a"].multiply(2)
     assert is_treeclass_equal(lhs, rhs)
 
-    # by param
+    #
+    lhs = A(1, 4, jnp.array([1, 4, 9, 16, 25]))
+    rhs = init.at[init != "a"].apply(lambda x: x**2)
+    assert is_treeclass_equal(lhs, rhs)
+
+    lhs = A(1, 3, jnp.array([2, 3, 4, 5, 6]))
+    rhs = init.at[init != "a"].apply(lambda x: x + 1)
+    assert is_treeclass_equal(lhs, rhs)
+
+    lhs = A(1, 30, jnp.array([20, 30, 40, 50, 60]))
+    rhs = init.at[init != "a"].apply(lambda x: (x + 1) * 10)
+    assert is_treeclass_equal(lhs, rhs)
+
+    lhs = A(1, 3, jnp.array([2, 3, 4, 5, 6]))
+    rhs = init.at[init != "a"].add(1)
+    assert is_treeclass_equal(lhs, rhs)
+
+    lhs = A(1, 1, jnp.array([0.5, 1, 1.5, 2, 2.5]))
+    rhs = init.at[init != "a"].divide(2.0)
+    assert is_treeclass_equal(lhs, rhs)
+
+    # raise
     with pytest.raises(ValueError):
         init.freeze().at[init == "a"].apply(lambda x: x**2)
 
@@ -440,7 +525,13 @@ def test_apply_and_its_derivatives():
     with pytest.raises(ValueError):
         init.freeze().at[(init == "a") | (A == "b")].set(0)
 
-    @treeclass
+    with pytest.raises(ValueError):
+        init.freeze().at[(init == init) | (A == "b")].set(0)
+
+    with pytest.raises(ValueError):
+        init.freeze().at[(init == init) | (A == "b")].reduce_sum()
+
+    @pytc.treeclass
     class Test:
         a: float = field(metadata={"name": "a", "unit": "m"})
         b: float = field(metadata={"name": "b", "unit": "m"})
@@ -448,23 +539,31 @@ def test_apply_and_its_derivatives():
         d: jnp.ndarray = field(metadata={"name": "d", "unit": "m"})
         name: str
 
-    A = Test(10, 20, 30, jnp.array([1, 2, 3, 4, 5]), "A")
+    A = Test(10, 20, 30, jnp.array([1, 2, 3, 4, 5]), static_value("A"))
 
     B = A.at[A == {"name": "a"}].apply(lambda _: 100, array_as_leaves=False)
-    assert is_treeclass_equal(B, Test(100, 20, 30, jnp.array([1, 2, 3, 4, 5]), "A"))
+    assert is_treeclass_equal(
+        B, Test(100, 20, 30, jnp.array([1, 2, 3, 4, 5]), static_value("A"))
+    )
 
     B = A.at[(A == {"name": "a"}) | (A == {"name": "b"})].set(
         100, array_as_leaves=False
     )
-    assert is_treeclass_equal(B, Test(100, 100, 30, jnp.array([1, 2, 3, 4, 5]), "A"))
+    assert is_treeclass_equal(
+        B, Test(100, 100, 30, jnp.array([1, 2, 3, 4, 5]), static_value("A"))
+    )
 
     B = A.at[A == {"": ""}].apply(lambda _: 100, array_as_leaves=False)
-    assert is_treeclass_equal(B, Test(10, 20, 30, jnp.array([1, 2, 3, 4, 5]), "A"))
+    assert is_treeclass_equal(
+        B, Test(10, 20, 30, jnp.array([1, 2, 3, 4, 5]), static_value("A"))
+    )
 
     B = A.at[(A == {"name": "a"}) | (A == {"name": "b"}) | (A == {"name": "c"})].set(
         100, array_as_leaves=False
     )
-    assert is_treeclass_equal(B, Test(100, 100, 100, jnp.array([1, 2, 3, 4, 5]), "A"))
+    assert is_treeclass_equal(
+        B, Test(100, 100, 100, jnp.array([1, 2, 3, 4, 5]), static_value("A"))
+    )
 
     B = A.at[
         (A == {"name": "a"})
@@ -472,22 +571,27 @@ def test_apply_and_its_derivatives():
         | (A == {"name": "c"})
         | (A == {"name": "d"})
     ].apply(lambda _: 100, array_as_leaves=False)
-    assert is_treeclass_equal(B, Test(100, 100, 100, 100, "A"))
+    assert is_treeclass_equal(B, Test(100, 100, 100, 100, static_value("A")))
 
-    @treeclass
+    B = A.at[A != {"name": "a"}].apply(lambda _: 100)
+    assert is_treeclass_equal(
+        B, Test(10, 100, 100, jnp.array([100, 100, 100, 100, 100]), static_value("A"))
+    )
+
+    @pytc.treeclass
     class L0:
         a: int = field(default=1, metadata={"name": "a", "unit": "m"})
         b: int = 2
         c: int = 3
 
-    @treeclass
+    @pytc.treeclass
     class L1:
         a: int = field(default=1, metadata={"name": "a", "unit": "m"})
         b: int = 2
         c: int = 3
         d: L0 = L0()
 
-    @treeclass
+    @pytc.treeclass
     class L2:
         a: int = field(default=10, metadata={"name": "a", "unit": "m"})
         b: int = 20
@@ -499,9 +603,40 @@ def test_apply_and_its_derivatives():
     rhs = L2(100, 20, 30, L1(100, 2, 3, L0(100, 2, 3)))
     assert is_treeclass_equal(lhs, rhs)
 
+    t = L2()
+    lhs = t.at[t != {"name": "a"}].apply(lambda _: 100)
+    rhs = L2(10, 100, 100, L1(1, 100, 100, L0(1, 100, 100)))
+
+    assert is_treeclass_equal(lhs, rhs)
+
+    @pytc.treeclass
+    class L0:
+        a: int = 1
+        b: int = 2
+        c: int = 3
+
+    @pytc.treeclass
+    class L1:
+        a: int = 1
+        b: int = 2
+        c: int = 3
+        d: L0 = L0()
+
+    @pytc.treeclass
+    class L2:
+        a: int = 10
+        b: int = 20
+        c: int = 30
+        d: L1 = field(default=L1(), metadata={"name": "d", "unit": "m"})
+
+    t = L2()
+    lhs = t.at[t == {"name": "d"}].apply(lambda _: 100)
+    rhs = L2(10, 20, 30, L1(100, 100, 100, L0(100, 100, 100)))
+    assert is_treeclass_equal(lhs, rhs)
+
 
 def test_reduce():
-    @treeclass
+    @pytc.treeclass
     class A:
         a: int
         b: int
@@ -521,7 +656,7 @@ def test_reduce():
     rhs = init.at[init > 100].reduce(lambda x, y: x + jnp.sum(y))
     assert lhs == rhs
 
-    @treeclass
+    @pytc.treeclass
     class B:
         a: int
         b: int
@@ -539,7 +674,7 @@ def test_reduce():
 
 
 def test_reduce_and_its_derivatives():
-    @treeclass
+    @pytc.treeclass
     class Linear:
         weight: jnp.ndarray
         bias: jnp.ndarray
@@ -550,10 +685,10 @@ def test_reduce_and_its_derivatives():
             )
             self.bias = jnp.ones((1, out_dim))
 
-        def __call__(self, x):
-            return x @ self.weight + self.bias
+        # def __call__(self, x):
+        #     return x @ self.weight + self.bias
 
-    @treeclass
+    @pytc.treeclass
     class StackedLinear:
         l1: Linear = field(metadata={"description": "First layer"})
         l2: Linear = field(metadata={"description": "Second layer"})
@@ -564,12 +699,12 @@ def test_reduce_and_its_derivatives():
             self.l1 = Linear(key=keys[0], in_dim=in_dim, out_dim=hidden_dim)
             self.l2 = Linear(key=keys[2], in_dim=hidden_dim, out_dim=out_dim)
 
-        def __call__(self, x):
-            x = self.l1(x)
-            x = jax.nn.tanh(x)
-            x = self.l2(x)
+        # def __call__(self, x):
+        #     x = self.l1(x)
+        #     x = jax.nn.tanh(x)
+        #     x = self.l2(x)
 
-            return x
+        #     return x
 
     model = StackedLinear(in_dim=1, out_dim=1, hidden_dim=5, key=jax.random.PRNGKey(0))
 
@@ -590,3 +725,52 @@ def test_reduce_and_its_derivatives():
     assert (model.at[model == jnp.ndarray].reduce_max()) == 1.3969219
     assert (model.at[model == jnp.ndarray].reduce_sum()) == 3.3538322
     assert (model.at[model == jnp.ndarray].reduce_product()) == 0.84782064
+
+
+def test_not_implemented():
+    @pytc.treeclass
+    class Test:
+        a: int = 1
+
+    t = Test()
+
+    with pytest.raises(NotImplementedError):
+        _at_get(t == t, 3)
+
+    with pytest.raises(NotImplementedError):
+        _at_set(t == t, 3, 3)
+
+    with pytest.raises(NotImplementedError):
+        _at_reduce(t == t, lambda x: x, 3)
+
+    with pytest.raises(NotImplementedError):
+        _at_apply(t == t, lambda x: x, 3)
+
+    with pytest.raises(NotImplementedError):
+        _at_get(t == t, np.array([1, 2, 3]))
+
+    with pytest.raises(NotImplementedError):
+        _at_set(t == t, 3, np.array([1, 2, 3]))
+
+    with pytest.raises(NotImplementedError):
+        _at_reduce(t == t, lambda x: x, np.array([1, 2, 3]))
+
+    with pytest.raises(NotImplementedError):
+        _at_apply(t == t, lambda x: x, np.array([1, 2, 3]))
+
+
+def test_is_leaf():
+    @pytc.treeclass
+    class Test:
+        a: int
+
+    t = Test([1, 2, 3, None])
+
+    assert is_treeclass_equal(
+        t.at[...].set(10, is_leaf=lambda x: x is None), Test([10, 10, 10, 10])
+    )
+
+    assert is_treeclass_equal(
+        t.at[...].apply(lambda x: 10, is_leaf=lambda x: x is None),
+        Test([10, 10, 10, 10]),
+    )
