@@ -6,6 +6,7 @@ from pytreeclass.src.tree_util import (
     _node_count_and_size,
     is_treeclass,
     is_treeclass_leaf,
+    _dispatched_tree_map
 )
 
 
@@ -43,7 +44,7 @@ def test__node_count_and_size():
         b: int = 1
 
     t = Test()
-    assert _node_count_and_size(t.b) == (complex(0, 1), complex(28, 0))
+    assert _node_count_and_size(t.b) == (complex(0, 1), complex(0, 28))
     assert _node_count_and_size(t.a) == (complex(3, 0), complex(12, 0))
 
     assert _node_count_and_size(jnp.array([1, 2, 3, 4, 5])) == (
@@ -68,3 +69,11 @@ def test__node_count_and_size():
 
     with pytest.raises(ValueError):
         xx.test = 1
+
+    
+def test_dispatched_tree_map():
+
+    with pytest.raises(NotImplementedError):
+        class A : ... 
+        _dispatched_tree_map(lambda x,y: x, 1,A())
+
